from tinymce import TinyMCE

from gibe.model import Comment
@Comment.getContentHtml.when('self.content_format == "tinymce"')
def getContentHtmlPostMarkup(self):
    return self.content
@Comment.getExcerptHtml.when('self.content_format == "tinymce"')
def getExcerptHtmlPostMarkup(self):
    return self.content

from turbogears import widgets, validators
def addCommentFields(wl):
    class CommentFormFieldsExtra(widgets.WidgetsList):
        comment = TinyMCE(validator=validators.NotEmpty, label = "Comment",
            mce_options = dict(
                mode = "exact",
                theme = "advanced",
                plugins = "fullscreen",
                relative_urls = False,
                theme_advanced_buttons2_add = "fullscreen",
                extended_valid_elements = "a[href|target|name]",
                paste_auto_cleanup_on_paste = True,
                paste_convert_headers_to_strong = True,
                paste_strip_class_attributes = "all",
                theme_advanced_buttons3 = "",
                remove_linebreaks = False,
                browsers = "msie,gecko",
            ),
            rows=15,
            cols=60,
        )
    wl.extend(CommentFormFieldsExtra())

import commenting
from gibe.util import sanitise
class Commenting(commenting.Commenting):
    def convert(self, kw):
        kw['comment'] = sanitise(kw['comment']).decode('utf-8')



import cherrypy

import turbogears
from turbogears import widgets

from gibe import model
import routes

import logging
log = logging.getLogger("gibe.feeds")

from turbogears import feed
class FeedController(feed.FeedController):
    def get_feed_data(self, blog, **kw):
        host = cherrypy.config.get('gibe.hostname', cherrypy.request.headers.get('Host', ''))
        tagname = kw.get('tagname', '')
        if tagname:
            t = model.Tag.get_by(name=tagname)
            last_entries = t.recent_posts(limit=5)
        else:
            last_entries = model.Post.select(order_by=model.desc(model.Post.c.creation_time), limit = 5)

        blog_base = cherrypy.config.get("gibe.blog_base", routes.url_for("frontpage", host=host))

        return dict(
            title=blog.name,
            #author=dict(name=blog.owner.name),
            link=blog_base,
            updated=last_entries[0].creation_time,
            generator="gibe",
            entries=[
                dict(title=post.title, content=post.content, summary=post.excerpt, updated = post.modification_time, published = post.creation_time,
                #author=dict(name=post.author.name),
                link=routes.url_for('posts', post=post, host=host)) for post in last_entries],
        )

class FeedLink(widgets.Source):
    template = """
    <link rel="alternate" href="$src" title="$title" type="$type" />
    """
    params = ['title', 'type']
    params_doc = {'src': 'The feed URL', 'title': 'Title for the feed', 'type': 'MIME type for the feed'}

    retrieve_css = widgets.set_with_self

class FeedLinks(widgets.Widget):
    def retrieve_css(self):
        return [
            FeedLink(src=routes.url_for('rss2.0.xml'), title="RSS 2.0", type="application/rss+xml"), 
            FeedLink(src=routes.url_for('atom1_0'), title="Atom 1.0", type="application/atom+xml"), 
        ]

feedlinks = FeedLinks()




# Release information about gibe

version = "0.10.8"

description = "Blog using TurboGears with popular replacement components"
long_description = """TurboGears-based web log supporting multiple web logs on a single instance, using Genshi, SQLAlchemy, and Routes, and a plugin architecture."""
author = "Neil Blakey-Milner, et al"
email = "nbm@nxsy.org, gibe-developers@googlegroups.com" 
copyright = "2006-2007, Neil Blakey-Milner"

# if it's open source, you might want to specify these
url = "http://code.google.com/p/gibe/"
download_url = "http://code.google.com/p/gibe/downloads/list"
license = "BSD"



#!/home/nbm/fs/turbogears-1.0b1/bin/python
import pkg_resources
pkg_resources.require("TurboGears")

import turbogears
import cherrypy
cherrypy.lowercase_api = True

from os.path import *
import sys

# first look on the command line for a desired config file,
# if it's not on the command line, then
# look for setup.py in this directory. If it's not there, this script is
# probably installed
if len(sys.argv) > 1:
    turbogears.update_config(configfile=sys.argv[1], 
        modulename="gibe.config")
elif exists(join(dirname(__file__), "setup.py")):
    turbogears.update_config(configfile="dev.cfg",
        modulename="gibe.config")
else:
    turbogears.update_config(configfile="prod.cfg",
        modulename="gibe.config")

from turbogears.startup import startTurboGears
from gibe.controllers import Root, RootBlogController
from gibe.model import *

import cherrypy
cherrypy.root = Root()
startTurboGears()

User.table.create()
Group.table.create()
Permission.table.create()
Post.table.create()
Blog.table.create()
Visit.table.create()
VisitIdentity.table.create()
Tag.table.create()

poster_blog.create()
user_group.create()
group_permission.create()
post_tag.create()

transaction = session.create_transaction()
u = User(about="Vainglorious open source personality",
    display_name="Neil Blakey-Milner",
    email_address = "nbm@mithrandr.moria.org", password = "asdf",
    user_id = 1, user_name = "nbm")
session.save(u)
g = Group(group_name = "admin", display_name = "Admin")
session.save(g)
g.users.append(u)
b = Blog(blog_id = 1, owner_id = 1, name = "Cosmic Seriosity Balance",
    slug = "cosmic-seriosity-balance",
    tagline = "Maintaining the Cosmic Seriosity Balance")
session.save(b)
transaction.commit()
session.close()



import cherrypy

from turbogears import widgets

class Analytics(widgets.Widget):
    template = """
    <div py:strip="True" xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://genshi.edgewall.org/">

<script src="http://www.google-analytics.com/urchin.js" type="text/javascript">
</script>
<script type="text/javascript">
_uacct = "$analytics_key";
urchinTracker();
</script>
        
    </div>
    """
    params = ['analytics_key']
    params_doc = {'analytics_key': 'Google Analytics Key'}

    location = widgets.js_location.bodybottom

    def retrieve_javascript(self):
        if not self.analytics_key:
            return []
        else:
            return widgets.set_with_self(self)

    def add_for_location(self, location):
        return location == self.location

analytics_key = cherrypy.config.get("analytics.key", None)
analytics_js = Analytics(analytics_key = analytics_key)



import pkg_resources
from turbogears.widgets.base import CSSLink, JSLink, CSSSource, JSSource, \
    Widget, CoreWD, static, js_location, register_static_directory


register_static_directory('gibe.static',
                          pkg_resources.resource_filename(__name__,
                                                          "static")
                         )

class SyntaxHighlighter(Widget):
    """This widget includes the syntax highlighter js and css into your 
    rendered page to syntax-hightlight textareas named 'code'. The supported
    languages can be listed at the 'languages' __init__ parameter.
    """
    available_langs = set([
        'CSharp',
        'Css',
        'Delphi',
        'Java',
        'JScript',
        'Php',
        'Python',
        'Ruby',
        'Sql',
        'Vb',
        'Xml',
        ])
    css = [CSSLink(static,"sh/SyntaxHighlighter.css")]
    
    def __init__(self, languages=['Python', 'Xml', 'JScript', 'Php', 'Css']):
        javascript = [ 
            JSLink(static, 'sh/shCore.js', location=js_location.bodybottom)
            ]
        for lang in languages:
            if lang not in self.available_langs:
                raise ValueError, ("Unsupported language %s. Available "
                                   "languages: '%s'" % 
                                   (lang, ', '.join(self.available_langs)))
            source = "sh/shBrush%s.js" % lang
            javascript.append(
                JSLink(static, source, location=js_location.bodybottom)
                )
        javascript.append(
            JSLink('gibe.static', 'javascript/shTag.js', location=js_location.bodybottom)
        )
        javascript.append(
            JSSource(
                "dp.SyntaxHighlighter.HighlightTag('pre');",
                location=js_location.bodybottom
                )
            )
        self.javascript = javascript

syntaxhighlighter = SyntaxHighlighter()



"""
Post Markup v1.0.1
Author: Will McGugan (http://www.willmcgugan.com)
"""

import re
from urllib import quote, unquote, quote_plus
from urlparse import urlparse, urlunparse
from copy import copy


pygments_available = True
try:
    from pygments import highlight
    from pygments.lexers import get_lexer_by_name, ClassNotFound
    from pygments.formatters import HtmlFormatter
except ImportError:
    pygments_available = False


def Create(include=None, exclude=None, use_pygments=True):
    
    """
    Create a postmarkup object.
    include -- List or similar iterable containing the names of the tags to use
               If omitted, all tags will be used
    exclude -- List or similar iterable containing the names of the tags to exclude.
               If omitted, no tags will be excluded
    use_pygments -- If True, Pygments (http://pygments.org/) will be used for the code tag,
                    otherwise it will use <pre>code</pre>
    """
    
    markup = PostMarkup()
    
    def add_tag(name, tag_class, *args):
        if include is None or name in include:
            if exclude is not None and name in exclude:
                return
            markup.add_tag(name, tag_class, *args)
    
    add_tag('b', SimpleTag, 'b', 'strong')
    add_tag('i', SimpleTag, 'i', 'em')
    add_tag('u', SimpleTag, 'u', 'u')
    add_tag('s', SimpleTag, 's', 'strike')
    add_tag('link', LinkTag, 'link')
    add_tag('url', LinkTag, 'url')    
    add_tag('quote', QuoteTag)
    
    add_tag('wiki', SearchTag, 'wiki',
            "http://en.wikipedia.org/wiki/Special:Search?search=%s", 'Wikipedia')
    add_tag('google', SearchTag, 'google',
            "http://www.google.com/search?hl=en&q=%s&btnG=Google+Search", 'Google')
    add_tag('wiki', SearchTag, 'wiki',
            "http://www.reference.com/search?q=%s", 'Definition')
    
    if use_pygments:
        assert pygments_available, "Could Not import pygments (http://pygments.org/)"
        add_tag('code', PygmentsCodeTag, 'code')
    else:
        add_tag('code', SimpleTag, 'code', 'pre')
    
    return markup


re_html=re.compile('<.*?>|\&.*?\;')
def textilize(s):
    """Remove markup from html"""
    return re_html.sub("", s)    

re_excerpt = re.compile(r'\[".*?\]+?.*?\[/".*?\]+?', re.DOTALL)
re_remove_markup = re.compile(r'\[.*?\]', re.DOTALL)

def remove_markup(post):
    return re_remove_markup.sub("", post)

def get_excerpt(post):
    match = re_excerpt.search(post)
    if match is None:
        return ""
    excerpt = match.group(0)
    excerpt=excerpt.replace('\n', "<br/>")
    return remove_markup(excerpt)


class TagBase(object):    
    """
    Base class for a Post Markup tag.        
    """
    
    def __init__(self, name):
        self.name = name
        self.params = None
        self.auto_close = False
        self.enclosed = False
                
    def open(self, open_pos):
        """Called when the tag is opened. Should return a string or a stringifyable object."""
        self.open_pos = open_pos
        return ''
    
    def close(self, close_pos, content):
        """Called when the tag is closed. Should return a string or a stringifyable object."""
        self.close_pos = close_pos
        self.content = content
        return ''

    def get_tag_contents(self):
        """Gets the contents of the tag."""
        content_elements = self.content[self.open_pos+1:self.close_pos]
        contents = "".join([unicode(element) for element in content_elements if isinstance(element, StringToken)])
        contents = textilize(contents)
        return contents
    
    def get_raw_tag_contents(self):
        """Gets the raw contents (includes html tags) of the tag."""
        content_elements = self.content[self.open_pos+1:self.close_pos]
        contents = "".join(element.raw for element in content_elements)        
        return contents
    
# A proxy object that calls a callback when converted to a string
class TagStringify(object):
    def __init__(self, callback, raw):
        self.callback = callback
        self.raw = raw
    def __str__(self):
        return self.callback()
    def __unicode__(self):
        return self.callback()
    def __repr__(self):
        return self.__unicode__()


class SimpleTag(TagBase):
    
    """Simple substitution tag."""
    
    def __init__(self, name, substitute):
        TagBase.__init__(self, name)        
        self.substitute = substitute
    
    def open(self, open_pos):
        """Called to render the opened tag."""
        return "<%s>"%(self.substitute)
        
    def close(self, close_pos, content):
        """Called to render the closed tag."""
        return "</%s>"%(self.substitute)    


class LinkTag(TagBase):
    
    """Tag that generates a link (</a>)."""
    
    def __init__(self, name):
        TagBase.__init__(self, name)
        
    def open(self, open_pos):
        self.open_pos = open_pos
        return TagStringify(self._open, self.raw)
    
    def close(self, close_pos, content):
        
        self.close_pos = close_pos
        self.content = content        
        return TagStringify(self._close, self.raw)
    
    def _open(self):
        if self.params:
            url=self.params
        else:
            url=self.get_tag_contents()
        
        self.domain = ""
        #Unquote the url
        self.url=unquote(url)        
        
        #Disallow javascript links
        if "javascript:" in self.url.lower():            
            return ""
                
        #Disallow non http: links
        url_parsed = urlparse(self.url)
        if url_parsed[0] and url_parsed[0].lower()!='http':
            return ""
        
        #Prepend http: if it is not present
        if not url_parsed[0]:
            self.url="http://"+self.url
            url_parsed = urlparse(self.url)
            
        #Get domain       
        self.domain = url_parsed[1].lower()
        
        #Remove www for brevity
        if self.domain.startswith('www.'):
            self.domain = self.domain[4:]
        
        #Quote the url        
        self.url="http:"+urlunparse( map(quote, ("",)+url_parsed[1:]) )
        
        #Sanity check        
        if not self.url:
            return ""   
        
        if self.domain:
            return '<a href="%s">'%self.url
        else:
            return ""
        
    def _close(self):
        
        if self.domain:
            return '</a>'+self.annotate_link(self.domain)
        else:
            return ''        
    
    def annotate_link(self, domain):
        """Annotates a link with the domain name.
        Override this to disable or change link annotation.
        """
        return " [%s]"%domain


class QuoteTag(TagBase):
    """
    Generates a blockquote with a message regarding the author of the quote.
    """
    def __init__(self):
        TagBase.__init__(self, 'quote')
        
    def open(self, open_pos):
        return '<blockquote><em>%s</em><br/>'%(self.params)
        
    def close(self, close_pos, content):
        return "</blockquote>"


class SearchTag(TagBase):
    """
    Creates a link to a search term.
    """
    
    def __init__(self, name, url, label=""):
        TagBase.__init__(self, name)
        self.url=url
        self.search = ""
        self.label = label or name
    
    def __str__(self):
        
        link='<a href="%s">'%self.url
        
        if '%' in link:
            return link%quote_plus(self.get_tag_contents())
        else:
            return link

    def __unicode__(self):
        
        link='<a href="%s">'%self.url
        
        if '%' in link:
            return link%quote_plus(self.get_tag_contents())
        else:
            return link
    
    def open(self, open_pos):
        self.open_pos = open_pos
        return TagStringify(self._open, self.raw)
    
    def close(self, close_pos, content):
        
        self.close_pos = close_pos
        self.content = content        
        return TagStringify(self._close, self.raw)
    
    def _open(self):
        if self.params:
            search=self.params
        else:
            search=self.get_tag_contents()
        link='<a href="%s">'%self.url
        if '%' in link:
            return link%quote_plus(search)
        else:
            return link
        
    def _close(self):
        
        if self.label:
            return '</a>'+self.annotate_link(self.label)
        else:
            return ''
        
    def annotate_link(self, domain):
        return " [%s]"%domain   



class PygmentsCodeTag(TagBase):
    
    def __init__(self, name):
        TagBase.__init__(self, name)
        self.enclosed = True
    
    def open(self, open_pos):
        self.open_pos = open_pos
        return TagStringify(self._open, self.raw)
    
    def close(self, close_pos, content):
        
        self.close_pos = close_pos
        self.content = content        
        return TagStringify(self._close, self.raw)

    def _open(self):
        
        try:
            lexer = get_lexer_by_name(self.params, stripall=True)
        except ClassNotFound:
            return '<div style="code"><pre>%s</pre>'%postmarkup.Escape(self.get_raw_tag_contents())
        formatter = HtmlFormatter(linenos=False, cssclass="code")        
        code = self.get_raw_tag_contents()
        result = highlight(code, lexer, formatter)
        return result+"\n<div style='display:none'>"
    
    def _close(self):
        return "</div>"
            

# http://effbot.org/zone/python-replace.htm
class MultiReplace:

    def __init__(self, repl_dict):
        # "compile" replacement dictionary

        # assume char to char mapping
        charmap = map(chr, range(256))
        for k, v in repl_dict.items():
            if len(k) != 1 or len(v) != 1:
                self.charmap = None
                break
            charmap[ord(k)] = v
        else:
            self.charmap = string.join(charmap, "")
            return

        # string to string mapping; use a regular expression
        keys = repl_dict.keys()
        keys.sort() # lexical order
        keys.reverse() # use longest match first        
        pattern = "|".join(re.escape(key) for key in keys)
        self.pattern = re.compile(pattern)
        self.dict = repl_dict

    def replace(self, str):
        # apply replacement dictionary to string
        if self.charmap:
            return string.translate(str, self.charmap)
        def repl(match, get=self.dict.get):
            item = match.group(0)
            return get(item, item)
        return self.pattern.sub(repl, str)


class StringToken(object):
    
    def __init__(self, raw):
        self.raw = raw
    
    def __str__(self):
        return PostMarkup.standard_replace.replace(self.raw)

    def __unicode__(self):
        return PostMarkup.standard_replace.replace(self.raw)
    

def Escape(s):
    return PostMarkup.standard_replace.replace(s)

class PostMarkup(object):
    
    standard_replace = MultiReplace({'<':'&lt;', '>':'&gt;', '&':'&amp;', '\n':'<br/>'})
    
    TOKEN_TAG, TOKEN_PTAG, TOKEN_TEXT = range(3)
        
    
    @staticmethod
    def TagFactory(tag_class, *args):
        """
        Returns a callable that returns a new tag instance.
        """
        def make():
            return tag_class(*args)
        
        return make    
    
    
    # I tried to use RE's. Really I did.
    @staticmethod
    def tokenize(post):
        
        text = True
        pos = 0
        
        def find_first(post, pos, c):
            f1 = post.find(c[0], pos)
            f2 = post.find(c[1], pos)
            if f1 == -1:
                return f2
            if f2 == -1:
                return f1
            return min(f1, f2)
        
        while True:
            
            brace_pos = post.find('[', pos)
            if brace_pos == -1:
                yield PostMarkup.TOKEN_TEXT, post[pos:]
                return
            if brace_pos - pos > 0:
                yield PostMarkup.TOKEN_TEXT, post[pos:brace_pos]
                
            pos = brace_pos                        
            end_pos = pos+1
            
            end_pos = find_first(post, end_pos, ']=')            
            if end_pos == -1:
                yield PostMarkup.TOKEN_TEXT, post[pos:]
                return            
                
            if post[end_pos] == ']':
                yield PostMarkup.TOKEN_TAG, post[pos:end_pos+1]
                pos = end_pos+1
                continue
            
            if post[end_pos]=='=':            
                try:
                    end_pos += 1
                    while post[end_pos]==' ':
                        end_pos += 1
                    if post[end_pos]!='"':
                        end_pos = post.find(']', end_pos+1)
                        if end_pos == -1:
                            return
                        yield PostMarkup.TOKEN_TAG, post[pos:end_pos+1]
                    else:
                        end_pos = find_first(post, end_pos, '"]')
                        if end_pos==-1:
                            return                        
                        if post[end_pos] == '"':
                            end_pos = post.find('"', end_pos+1)
                            if end_pos == -1:
                                return
                            end_pos = post.find(']', end_pos+1)
                            if end_pos == -1:
                                return
                            yield PostMarkup.TOKEN_PTAG, post[pos:end_pos+1]
                        else:
                            yield PostMarkup.TOKEN_TAG, post[pos:end_pos+1] 
                    pos = end_pos+1
                except IndexError:
                    return
                
                
    def __init__(self):
        
        self.tags={}
    
    
    def default_tags(self):        
        """
        Sets up a minimal set of tags.
        """
        self.tags['b'] = PostMarkup.TagFactory(SimpleTag, 'b', 'strong')
        self.tags['i'] = PostMarkup.TagFactory(SimpleTag, 'i', 'em')
        self.tags['u'] = PostMarkup.TagFactory(SimpleTag, 'u', 'u')
        self.tags['s'] = PostMarkup.TagFactory(SimpleTag, 's', 'strike')
        
        return self


    def add_tag(self, name, tag_class, *args):
        """
        Add a tag factory to the markup.
        """
        self.tags[name] = PostMarkup.TagFactory(tag_class, *args)
        

    def __call__(self, post_markup):
        return self.render_to_html(post_markup)


    def render_to_html(self, post_markup, max_length = None, continuation_text = None, max_continuous = None):
        """
        Converts Post Markup to xhtml.
        """        
        
        post=[]
        tag_stack=[]        
        break_stack=[]
        enclosed = False

        remaining_length = max_length
        continued = False
        
        def check_tag_stack(tag_name):
            """Check to see if a tag has been opened."""
            for tag in reversed(tag_stack):
                if tag_name == tag.name:
                    return True
            return False
        
        def redo_break_stack():
            """Re-opens tags that have been closed prematurely."""
            while break_stack:
                tag= copy(break_stack.pop())
                tag.raw=""
                tag_stack.append(tag)
                post.append(tag.open(len(post)))
        
        for tag_type, tag_token in PostMarkup.tokenize(post_markup):
            #print tag_type, tag_token
            raw_tag_token = tag_token
            if tag_type == PostMarkup.TOKEN_TEXT:
                redo_break_stack()                                    
                if max_continuous:
                    orig_tag_token = tag_token
                    words = tag_token.split(" ")
                    out_words = []
                    for word in words:
                        while len(word) > max_continuous:
                            out_words.append(word[:max_continuous])
                            word = word[max_continuous:]
                        out_words.append(word)
                    tag_token = " ".join(out_words)
                        
                if max_length:
                   if len(tag_token) > remaining_length:
                       post.append(StringToken(tag_token[:remaining_length]))
                       continued = True
                       break
                   remaining_length -= len(tag_token)
                post.append(StringToken(tag_token))
                continue
            elif tag_type == PostMarkup.TOKEN_TAG:
                tag_token = tag_token[1:-1].lstrip()
                if ' ' in tag_token:
                    tag_name, tag_attribs = tag_token.split(' ', 1)
                    tag_attribs = tag_attribs.strip()
                else:
                    tag_name = tag_token
                    tag_attribs = ""
            else:
                tag_token = tag_token[1:-1].lstrip()
                tag_name, tag_attribs = tag_token.split('=', 1)
                tag_attribs=tag_attribs.strip()[1:-1]

            tag_name = tag_name.strip().lower()
            
            end_tag = False
            if tag_name.startswith('/'):
                end_tag = True
                tag_name = tag_name[1:]
            
            if not end_tag:
                if enclosed:
                    post.append(StringToken(raw_tag_token))
                    continue                    
                if tag_name not in self.tags:                    
                    continue                    
                tag = self.tags[tag_name]()
                enclosed = tag.enclosed
                tag.raw = raw_tag_token
                
                redo_break_stack()
                tag.params=tag_attribs
                tag_stack.append(tag)                    
                post.append(tag.open(len(post)))
                if tag.auto_close:
                    end_tag = True
                
            if end_tag:
                if not check_tag_stack(tag_name):
                    if enclosed:
                        post.append(StringToken(raw_tag_token))
                    continue                
                enclosed = False 
                while tag_stack[-1].name != tag_name:
                    tag = tag_stack.pop()
                    break_stack.append(tag)
                    if not enclosed:
                        post.append(tag.close(len(post), post))                
                post.append(tag_stack.pop().close(len(post), post))
                
        if tag_stack:            
            redo_break_stack()
            while tag_stack:            
                post.append(tag_stack.pop().close(len(post), post))

        if continued and continuation_text:
            post.append(continuation_text)
                     
        return "".join(unicode(p) for p in post)
                
                

def test():
    
    #post_markup = PostMarkup()
    post_markup = Create()
        
    tests = []
    print """<link rel="stylesheet" href="code.css" type="text/css" />\n"""
    tests.append("[b]Hello[/b]")
    tests.append("[s]Strike through[/s]")
    tests.append("[b]bold [i]bold and italic[/b] italic[/i]")
    tests.append("[google]Will McGugan[/google]")
    tests.append("[wiki Will McGugan]Look up my name in Wikipedia[/wiki]")
    tests.append("[link http://www.willmcgugan.com]My homepage[/link]")
    tests.append("[link]http://www.willmcgugan.com[/link]")
    tests.append("[quote Will said...]BBCode is very cool[/quote]")

    tests.append("""[code python]
# A proxy object that calls a callback when converted to a string
class TagStringify(object):
    def __init__(self, callback, raw):
        self.callback = callback
        self.raw = raw
    def __str__(self):
        return self.callback()
    def __repr__(self):
        return self.__str__()
[/code]""")

    long_test="""[b]Long test[/b]

New lines characters are converted to breaks. Tags my be [b]ove[i]rl[/b]apped[/i].

[i]Open tags will be closed.
"""

    tests.append(long_test)

    for test in tests:        
        print "<pre>%s</pre>"%test
        print "<p>%s</p>"%post_markup(test)
        print "<br/>"
        print
        
if __name__ == "__main__":
    test()



import pkg_resources
import cherrypy
from turbogears import widgets, validators, error_handler, validate, flash
from turbogears.database import session

class Commenting(object):
    def convert(self, kw):
        pass
    def post_save(self, kw):
        pass

format_registry = {}
for comment_format_mod in pkg_resources.iter_entry_points("gibe.comment_formats"):
    mod = comment_format_mod.load()
    format_registry[comment_format_mod.name] = mod

def fields():
    formats = []

    formats_to_check = [
        cherrypy.config.get('gibe.comment_format.preferred'),
        cherrypy.config.get('gibe.comment_format.fallback', None),
        ['postmarkup', 'tinymce'],
    ]

    for fs in formats_to_check:
        if isinstance(fs, (str, unicode)):
            fs = [fs]
        for f in fs:
            if f not in formats:
                formats.append(f)

    for format in formats:
        if format in format_registry:
            class CommentFormFields(widgets.WidgetsList):
                postid = widgets.HiddenField()

                content_format = widgets.HiddenField(default=format,
                    validator=validators.OneOf(format_registry.keys()))

                name = widgets.TextField(validator=validators.NotEmpty,
                    label = "Your name", attrs=dict(size=60))

                email = widgets.TextField(label = "Email",
                    validator=validators.All(validators.Email, validators.NotEmpty),
                    attrs=dict(size=60))

                website = widgets.TextField(validator=validators.URL,
                    label = "Site", attrs=dict(size=60))

            comment_form_fields = CommentFormFields()

            format_registry[format].addCommentFields(comment_form_fields)
            return comment_form_fields

comment_form = widgets.TableForm(fields=fields(), submit_text="Post")

akismet = None
try:
    from akismet import Akismet
    if cherrypy.config.get("akismet.key"):
        akismet = Akismet(
            agent="Gibe",
            key=cherrypy.config.get("akismet.key"),
            blog_url=cherrypy.config.get("akismet.url")
        )
        akismet.verify_key()
except (ImportError, URLError), e:
    pass

turbomail = None
try:
    if cherrypy.config.get('mail.on', False) != False:
        import turbomail
except:
    pass

import routes
from gibe.model import Post, Comment, User

from datetime import datetime, timedelta

class CommentController(object):
    def post(self, blog, **kw):
        from gibe.controllers import gibe_controllers
        return gibe_controllers['self'].post(blog, **kw)

    @error_handler(post)
    @validate(form=comment_form)
    def add_comment(self, blog, **kw):
        post = Post.get_by(post_id = kw['postid'])
        if not post.accept_comments:
            flash("Post does not allow comments")
            raise routes.redirect_to('posts', post = post)

        content_format = kw['content_format']
        commenting = format_registry[content_format].Commenting()

        commenting.convert(kw)

        ckw = {
            'post_id': kw['postid'], 
            'blog_id': blog.blog_id, 
            'author_name': kw['name'],
            'author_url': kw['website'],
            'author_email': kw['email'],
            'content': kw['comment'],
            'approved': True,
            'posted_time': datetime.now(),
            'content_format': kw['content_format'],
        }

        c = Comment(**ckw)

        commenting.post_save(kw)

        self._check_for_spam(blog, c, kw)

        try:
            c.getContentHtml()
            c.getExcerptHtml()
        except:
            flash("Could not render comment, marked for moderation")
            c.approved = False
            session.save(c)

        raise routes.redirect_to('posts', post = post)

    def _check_for_spam(self, blog, c, kw):
        data = {}
        data['comment_author'] = kw['name'].encode('utf-8')
        data['comment_author_email'] = kw['email'].encode('utf-8')
        if kw['website']:
            data['comment_author_url'] = kw['website'].encode('utf-8')

        data['user_ip'] = cherrypy.request.headers.get('Remote-Addr')
        data['user_agent'] = cherrypy.request.headers.get('User-Agent', '')

        try:
            if akismet and akismet.comment_check(kw['comment'].encode('utf-8'), data):
                c.approved = False
        except IOError:
            # When approval is around, set .approved to False
            c.approved = False

        session.save(c)
        if c.approved == False:
            flash("Comment marked for moderation")
        else:
            flash("Comment added")

        if turbomail:
            try:
                #from_address = kw['email']
                to_address = User.get_by(user_id = blog.owner_id).email_address
                from_address = to_address
                if c.approved == False:
                    subject = "A possible spam comment awaits moderation on post %s" % (post.title)
                else:
                    subject = "A comment was made on post %s" % (post.title)
                message = turbomail.Message(from_address, to_address, subject)
                message.plain = u"""
    Post by %(name)s, email address %(email)s, site %(website)s

    Comment:

    %(comment)s
    """ % kw
                message.plain = message.plain.encode('utf-8', 'replace')
                turbomail.enqueue(message)
            except Exception:
                pass

commentController = CommentController()




from datetime import datetime

from sqlalchemy import *
from sqlalchemy.ext.activemapper import ActiveMapper, column, \
                                    one_to_many, one_to_one, many_to_many

from turbogears import identity 
from turbogears.database import metadata, session

import dispatch

# from identitymodel import Visit, VisitIdentity, Group, Permission, user_group, group_permission

poster_blog = Table("poster_blog", metadata,
                      Column("user_id", Integer,
                              ForeignKey("user.user_id"),
                              primary_key=True),
                      Column("blog_id", Integer,
                              ForeignKey("blog.blog_id"),
                              primary_key=True))

post_tag = Table("post_tag", metadata,
                      Column("post_id", Integer,
                              ForeignKey("post.post_id"),
                              primary_key=True),
                      Column("tag_id", Integer,
                              ForeignKey("tag.tag_id"),
                              primary_key=True))

from elementtree import ElementTree as ET
import operator

class Blog(ActiveMapper):
    class mapping:
        blog_id = column(Integer, primary_key=True)
        owner_id = column(Integer, foreign_key=ForeignKey('user.user_id'))

        name    = column(Unicode(255), unique=True)
        tagline = column(Unicode(255))
        slug = column(Unicode(255))
        theme = column(Unicode(255))

        #owner = relation('User', backref="owned_blogs")
        #posters = many_to_many("User", poster_blog, backref="blogs")
        posts = one_to_many("Post", backref="blog")

        # tags = many_to_many("Tag", blog_tag, backref="blogs")

    def months(self):
        s = select(["YEAR(creation_time) AS year, MONTH(creation_time) AS month"],
        from_obj = [Post.table], distinct=True, order_by=[desc("year"),
        desc("month")])
        c = s.execute()
        for year, month in c:
            yield datetime(year=year, month=month, day=1)
        
    def recent_posts(self, num = 5):
        posts = Post.select(Post.c.blog_id == self.blog_id, order_by=desc(Post.c.creation_time), limit = num)
        # posts.reverse()
        return posts

    def recent_comments(self, num = 5):
        comments = Comment.select((Comment.c.blog_id == self.blog_id) & (Comment.c.approved == True), order_by=desc(Comment.c.posted_time), limit = num)
        return comments

    def all_tags_count(self, min = None, limit = None):
        s = select([post_tag.c.tag_id, func.count(post_tag.c.post_id)], group_by=[post_tag.c.tag_id], order_by=[desc(func.count(post_tag.c.post_id))])
        if limit:
            s.limit = limit
        if min:
            s.having = func.count(post_tag.c.post_id) > min
        r = s.execute()
        for tag_id, num in r:
            yield Tag.get(tag_id), num

    def most_commented(self, num = 5, min = None):
        s = select([Post.c.post_id], Post.c.post_id == Comment.c.post_id, group_by=[Post.c.post_id], order_by=[desc(func.count(Comment.c.comment_id))])
        s.limit = num
        r = s.execute()
        for post_id, in r:
            yield Post.get(post_id)

class Post(ActiveMapper):
    class mapping:
        post_id = column(Integer, primary_key=True)
        blog_id = column(Integer, foreign_key=ForeignKey('blog.blog_id'))
        author_id = column(Integer, foreign_key=ForeignKey('user.user_id'))
        title = column(Unicode(255))
        content = column(Unicode())
        published = column(Boolean, default=False)
        creation_time = column(DateTime)
        modification_time = column(DateTime)
        trackback_urls = column(String)
        slug = column(Unicode(255))
        excerpt = column(Unicode())
        accept_comments = column(Boolean, default=True)
        content_format = column(Unicode(25), default = "html")
        excerpt_format = column(Unicode(25), default = "html")
        manual_excerpt = column(Boolean, default = False)
        
        comments = one_to_many("Comment", backref="post")
        # trackbacks = one_to_many("Trackback", backref="post")
        tags = many_to_many("Tag", post_tag, backref="posts")
        #blog = relation("Blog", backref="posts")

    def approved_comments(self):
        return Comment.select((Comment.c.post_id == self.post_id) & (Comment.c.approved == True))
        
    approved_comments = property(approved_comments)
        
    @dispatch.generic()
    def getContentHtml(self):
        pass

    @getContentHtml.when('self.content_format == "html"')
    def getContentHtmlfromHtml(self):
        return self.content

    @dispatch.generic()
    def getExcerptHtml(self):
        pass

    @getExcerptHtml.when('self.excerpt_format == "html"')
    def getExcerptHtmlfromHtml(self):
        return self.excerpt

    content_html = property(getContentHtml)
    excerpt_html = property(getExcerptHtml)

    def surrounding_posts(self, num = 2, numbefore = None):
        if not numbefore:
            numbefore = num

        posts = Post.select(Post.c.creation_time < self.creation_time, order_by=desc(Post.c.creation_time), limit = numbefore)
        posts += [self]
        posts += Post.select(Post.c.creation_time > self.creation_time, order_by=asc(Post.c.creation_time), limit = num) 

        posts.sort(key=operator.attrgetter("creation_time"))

        return posts

class Tag(ActiveMapper):
    class mapping:
        tag_id = column(Integer, primary_key=True)
        name = column(Unicode(255))
        display_name = column(Unicode(255))

    def recent_posts(self, limit = 5, offset = 0):
        posts = Post.select((Post.c.post_id == post_tag.c.post_id) &
            (post_tag.c.tag_id == self.tag_id),
            order_by=desc(Post.c.creation_time), limit = limit, offset =
            offset)
        return posts

class Comment(ActiveMapper):
    class mapping:
        comment_id = column(Integer, primary_key=True)
        post_id = column(Integer, foreign_key=ForeignKey('post.post_id'))
        blog_id = column(Integer, foreign_key=ForeignKey('blog.blog_id'))
        author_id = column(Integer, foreign_key=ForeignKey('user.user_id'))
        author_name = column(Unicode(255))
        author_url = column(String(255))
        author_email = column(String(255))
        content = column(Unicode())
        posted_time = column(DateTime)
        approved = column(Boolean, default=False)
        content_format = column(Unicode(25), default = "html")

    def getReference(self):
        if not self.author_url:
            span = ET.Element('span')
            span.text = self.author_name
            return span

        a = ET.Element('a')
        a.attrib['href'] = self.author_url
        a.attrib['rel'] = 'nofollow'
        a.text = self.author_name
        return a

    allowed_tags = {
        'a': ['title','href'],
        'abbr': ['title'],
        'acronym': ['title'],
        'b': [],
        'blockquote': [],
        'code': [],
        'em': [],
        'i': [],
        'strike': [],
        'strong': [],
    }

    def cleanContent(self):
        return True
    
    @dispatch.generic()
    def getContentHtml(self):
        pass

    @getContentHtml.when('self.content_format == "html"')
    def getContentHtmlfromHtml(self):
        return self.content

    @dispatch.generic()
    def getExcerptHtml(self):
        pass

    @getExcerptHtml.when('self.content_format == "html"')
    def getExcerptHtmlfromHtml(self):
        return self.content

### Identity models ####################################################

class Visit(ActiveMapper):
    class mapping:
        __table__ = "visit"
        visit_key = column(String(40), primary_key=True)
        created = column(DateTime, nullable=False, default=datetime.now)
        expiry = column(DateTime)

    def lookup_visit(cls, visit_key):
        return Visit.get(visit_key)
    lookup_visit = classmethod(lookup_visit)

# tables for SQLAlchemy identity
user_group = Table("user_group", metadata, 
                      Column("user_id", Integer,
                              ForeignKey("user.user_id"),
                              primary_key=True),
                      Column("group_id", Integer,
                              ForeignKey("group.group_id"),
                              primary_key=True))

group_permission = Table("group_permission", metadata,
                            Column("group_id", Integer,
                                    ForeignKey("group.group_id"),
                                    primary_key=True),
                            Column("permission_id", Integer,
                                ForeignKey("permission.permission_id"),
                                    primary_key=True))

class User(ActiveMapper):
    class mapping:
        __table__ = "user"
        user_id = column(Integer, primary_key=True)
        user_name = column(Unicode(16), unique=True)
        email_address = column(Unicode(255), unique=True)
        display_name = column(Unicode(255))
        password = column(Unicode(40))
        created = column(DateTime, default=datetime.now)

        avatar = column(TEXT)
        about = column(TEXT)

        #groups = many_to_many("Group", user_group, backref="users")
        #blogs = many_to_many("Blog", poster_blog, backref="posters")
        owned_blogs = one_to_many("Blog", backref="owner")

    def permissions(self):
        perms = set()
        for g in self.groups:
            perms = perms | set(g.permissions)
        return perms
    permissions = property(permissions)



class VisitIdentity(ActiveMapper):
    class mapping:
        __table__ = "visit_identity"
        visit_key = column(String(40), # foreign_key="visit.visit_key",
                          primary_key=True)
        user_id = column(Integer, foreign_key="user.user_id", index=True)


class Group(ActiveMapper):
    """
    An ultra-simple group definition.
    """
    class mapping:
        __table__ = "group"
        group_id = column(Integer, primary_key=True)
        group_name = column(Unicode(16), unique=True)
        display_name = column(Unicode(255))
        created = column(DateTime, default=datetime.now)

        users = many_to_many("User", user_group, backref="groups")
        permissions = many_to_many("Permission", group_permission,
                                   backref="groups")

class Permission(ActiveMapper):
    class mapping:
        __table__ = "permission"
        permission_id = column(Integer, primary_key=True)
        permission_name = column(Unicode(16), unique=True)
        description = column(Unicode(255))

        #groups = many_to_many("Group", group_permission,
        #                      backref="permissions")



import logging
log = logging.getLogger('gibe.util')
import re

def slugify(title, table, id=0):
    """
    Code taken from Toasty Goat project
    This ugly sack of crap returns a slug from a title. The title and
    table (Posts, Pages, etc) must be passed to determine if the slug
    already exists. The id is for editing a sluggable item, making it
    so that you may replace the item with the same slug without a
    problem.
    """
    regex = re.compile("[^\w\-\ ]")
    slug = regex.sub('', title).lower().strip().replace(' ', '-')

    append = ''
    while True:
        new_slug = slug[:(255 - len(str(append)))] + str(append)
        if not slug_exists(new_slug, table, id):
            return new_slug
        else:
            if append == '':
                append = 1

            append = append + 1


def slug_exists(slug, table, id):
    a = table.get_by(slug=slug)
    if not a:
        return False
    if a.post_id == id:
        return False
    return True

from elementtree import ElementTree as ET
from genshi.core import Attrs, Namespace, stripentities
from genshi.core import END, END_NS, START, START_NS, TEXT
from genshi.input import HTML
from genshi.filters import HTMLSanitizer


class Excerptifier:
    found_excerpt = False
    def __call__(self, stream, ctxt = None):
        open_tags = []
        for kind,data,pos in stream:
            if kind == START:
                tag, attrs = data
                open_tags.append(tag)
            if kind == END:
                open_tags.pop()
            if kind == START:
                tag, attrs = data
                if tag == "hr":
                    for name, value in attrs:
                        if name == "class" and value == "excerpt":
                            self.found_excerpt = True
                            break
                    if self.found_excerpt:
                        break
                    
            yield kind, data, pos

        if open_tags:
            for t in open_tags:
                yield END, t, pos

        raise StopIteration

def excerptify(post):
    html = HTML(post.content)
    e = Excerptifier()
    res = (html | e)
    foo = res.render(encoding=None)
    if e.found_excerpt:
        return foo
    return post.content

def commaise(elements):
    level = 0
    started = False
    for kind, data, pos in elements:
        if level == 0 and started:
            yield TEXT, ", ", pos

        if kind == START:
            started = True
            level += 1

        if kind == END:
            level -= 1

        yield kind, data, pos

def sanitise(htmltext):
    html = HTML(htmltext)
    return (html | HTMLSanitizer()).render()

from datetime import datetime, timedelta

def display_entries(posts):
    display_entries = []
    last = None
    for entry in posts:
	this = datetime(year=entry.creation_time.year, month=entry.creation_time.month, day=entry.creation_time.day)
	if last != this:
	    last = this
	    last_list = (last, [])
	    display_entries.append(last_list)

	last_list[1].append(entry)
    
    return display_entries

template_loader_installed = False
def install_template_loader():
    from genshi.template.loader import TemplateLoader, TemplateNotFound
    import cherrypy
    from turbogears import view
    global template_loader_installed

    if template_loader_installed:
        return

    template_loader_installed = True
    class MyTemplateLoader(TemplateLoader):
        def search_path(self):
            from gibe import plugin
            plugin_data = plugin.get_plugin_data(cherrypy.request.blog)
            return plugin_data['template_search_path']
        def dummy(self, value):
            pass
        search_path = property(search_path, dummy)

    old_loader = view.engines['genshi'].loader
    view.engines['genshi'].loader = MyTemplateLoader(search_path =
        old_loader.search_path, auto_reload = old_loader.auto_reload,
        default_encoding = old_loader.default_encoding, default_class =
        old_loader.default_class)
    old_loader = view.engines['genshi-markup'].loader
    view.engines['genshi-markup'].loader = MyTemplateLoader(search_path =
        old_loader.search_path, auto_reload = old_loader.auto_reload,
        default_encoding = old_loader.default_encoding, default_class =
        old_loader.default_class)




from turbogears import widgets
from turbogears import validators
import pkg_resources

import logging
log = logging.getLogger("gibe.plugin")

class Plugin(object):
    def initialise(self):
        return True

    def enabled(self):
        return True

    def post_top_widgets(self, blog, widgets, context):
        # widgets to put above the post
        pass

    def post_bottom_widgets(self, blog, widgets, context):
        # widgets to put below the post
        pass

    def post_list_top_widgets(self, blog, widgets, context):
        # widgets to put above each post in a list of posts
        pass

    def post_list_bottom_widgets(self, blog, widgets, context):
        # widgets to put below each post in a list of posts
        pass

    def blog_header_widgets(self, blog, widgets, context):
        # widgets (JS/CSS/Link) to put in the headers
        pass

    def blog_top_widgets(self, blog, widgets, context):
        # widgets (usually JS) to put at the top of the page body
        pass

    def blog_bottom_widgets(self, blog, widgets, context):
        # widgets (usually JS) to put at the bottom of the page body
        pass

    def post_edit_widgets(self, blog, widgets, context):
        # widgets to add to post create/edit admin page
        pass

    def post_edit_values(self, blog, post, values, context):
        # values for widgets in post create/edit admin page
        pass

    def post_edit_params(self, blog, post, params, context):
        # params for widgets in post create/edit admin page
        pass

    def post_edit_handle(self, blog, post, form_values, context):
        # post might not exist yet
        pass

    def post_edit_handle_after(self, blog, post, form_values, context):
        # post will exist now
        pass

    def map_routes(self, blog, mapper, controllers):
        pass

    def template_search_path(self, blog, search_path):
        pass

    def template_data(self, blog, template_data):
        pass

    def template_replacements(self, blog, template_replacements):
        pass

class MemberWidgetList(widgets.CompoundWidget):
    def __init__(self, widgets):
        super(MemberWidgetList, self).__init__()
        self.member_widgets = []
        self.member_widgets.extend(widgets)

    def __iter__(self):
        return self.member_widgets.__iter__()

    iter_member_widgets = __iter__

def create_mangle_func(mangler_name):
    def mangle_func(blog, context):
        widget_list = []
        for k, v in plugins.items():
            getattr(v, mangler_name)(blog, widget_list, context)
        return MemberWidgetList(widget_list)

    return mangle_func

post_top_widgets = create_mangle_func('post_top_widgets')
post_bottom_widgets = create_mangle_func('post_bottom_widgets')
blog_header_widgets = create_mangle_func('blog_header_widgets')
blog_top_widgets = create_mangle_func('blog_top_widgets')
blog_bottom_widgets = create_mangle_func('blog_bottom_widgets')
post_list_top_widgets = create_mangle_func('post_list_top_widgets')
post_list_bottom_widgets = create_mangle_func('post_list_bottom_widgets')

def post_edit_values(blog, post, context):
    values = {}
    if post:
        values['title'] = post.title
        values['content'] = post.content

    for k, v in plugins.items():
        v.post_edit_values(blog, post, values, context)
    return values

def post_edit_params(blog, post, context):
    params = {}

    for k, v in plugins.items():
        v.post_edit_params(blog, post, params, context)
    return params

from tinymce import TinyMCE
from turbojson.jsonify import encode

class TinyMCEWithPlugins(TinyMCE):
    params_doc = {
        'plugins': 'list of tuples of plugin name and plugin location',
    }
    params = params_doc.keys()
    plugins = []
    def update_params(self, d):
        super(TinyMCEWithPlugins, self).update_params(d)
        for plugin_name, plugin_url in d['plugins']:
            d['TinyMCEInit'] += '\ntinyMCE.loadPlugin(%s, %s);' % (encode(plugin_name), encode(plugin_url),)

def post_edit_widgets(blog, context):
    import turbogears
    class PostFormFields(widgets.WidgetsList):
        title = widgets.TextField(validator=validators.NotEmpty,
            label = "Title", attrs=dict(size="30"))
        content = TinyMCEWithPlugins(validator=validators.NotEmpty, label = "Content",
            mce_options = dict(
                mode = "exact",
                theme = "advanced",
                plugins = "fullscreen,insertexcerptline",
                relative_urls = False,
                theme_advanced_buttons2_add = "fullscreen,insertexcerptline",
                extended_valid_elements = "a[href|target|name]",
                paste_auto_cleanup_on_paste = True,
                paste_convert_headers_to_strong = True,
                paste_strip_class_attributes = "all",
                theme_advanced_buttons3 = "",
                remove_linebreaks = False,
                convert_newlines_to_brs = False,
                force_br_newlines = False,
            ),
            rows=25,
            cols=65,
            plugins=[
                ('insertexcerptline', turbogears.url('/tg_widgets/gibe.static/javascript/insertexcerptline')),
            ],
        )

    widgets_list = []
    for w in PostFormFields():
        widgets_list.append(w)

    for k, v in plugins.items():
        v.post_edit_widgets(blog, widgets_list, context)

    return widgets_list
    return PostFormFields()

def map_routes(blog, mapper, controllers):
    for k, v in plugins.items():
        v.map_routes(blog, mapper, controllers)

def template_data(blog):
    d = {}
    for k, v in plugins.items():
        v.template_data(blog, d)
    return d

def template_search_path(blog):
    search_path = []
    import gibe.templates
    search_path.extend(gibe.templates.__path__)
    for k, v in plugins.items():
        v.template_search_path(blog, search_path)
    return search_path

def post_edit_handle_after(blog, post, kw, context):
    for k, v in plugins.items():
        v.post_edit_handle_after(blog, post, kw, context)

def template_replacements(blog):
    d = {}
    for k, v in plugins.items():
        v.template_replacements(blog, d)
    return d

def find_template(blog, template):
    d = get_plugin_data(blog)
    template = d['template_replacements'].get(template, template)
    return template

def get_plugin_data(blog):
    from gibe import plugin
    blogid = blog.blog_id
    if blogid not in blog_plugin_data:
        blog_plugin_data[blogid] = {
            'post_top_widgets': plugin.post_top_widgets(blog, {}),
            'post_bottom_widgets': plugin.post_bottom_widgets(blog, {}),
            'post_list_top_widgets': plugin.post_list_top_widgets(blog, {}),
            'post_list_bottom_widgets': plugin.post_list_bottom_widgets(blog, {}),
            'post_edit_widgets': plugin.post_edit_widgets(blog, {}),
            'template_search_path': plugin.template_search_path(blog),
            'template_search_path': plugin.template_search_path(blog),
            'template_replacements': plugin.template_replacements(blog),
        }
        blog_plugin_data[blogid].update(plugin.template_data(blog))
    return blog_plugin_data[blogid]

def add_plugin_data(template=None):
    def wrap(func):
        def entangle(self, *args, **kw):
            ret = func(self, *args, **kw)

            blog = kw.get('blog', None)
            if not blog:
                return ret

            if not isinstance(ret, dict):
                return ret

            ret.update(get_plugin_data(blog))

            ret['blog'] = blog

            if template:
                ret['tg_template'] = find_template(blog, template)

            return ret
        return entangle
    return wrap


blog_plugin_data = {}
plugins = {}

def on_startup():
    for plugin_mod in pkg_resources.iter_entry_points("gibe.plugins"):
        plugin_name = plugin_mod.module_name
        if plugin_mod.attrs:
            plugin_name += ':' + '.'.join(plugin_mod.attrs)

        mod = plugin_mod.load()
        p = mod()
        if p.enabled():
            initialised = False
            try:
                initialised = p.initialise()
            except (Exception,), e:
                print e
                pass

            if initialised:
                plugins[plugin_name] = p

from turbogears.startup import call_on_startup
call_on_startup.append(on_startup)



import postmarkup

class CodeTag(postmarkup.TagBase):
    """
    Generates a blockquote with a message regarding the author of the quote.
    """
    def __init__(self):
        postmarkup.TagBase.__init__(self, 'code')

    def open(self, open_pos):
        if self.params == "python":
            return '<pre class="dp::syntaxhighlight python">'
        return '<pre class="code">'

    def close(self, close_pos, content):
        return "</pre>" 

from gibe.model import Comment
@Comment.getContentHtml.when('self.content_format == "postmarkup"')
def getContentHtmlPostMarkup(self):
    return pm.render_to_html(self.content)

@Comment.getExcerptHtml.when('self.content_format == "postmarkup"')
def getExcerptHtmlPostMarkup(self, length = 150):
    content = self.content
    #if len(content) > length:
    #    content = content[:length-3] + "..."
    return pm.render_to_html(content, max_length = length, continuation_text = "...", max_continuous = 25)

    input_content = self.content
    output_content = ""
    done = False
    remaining_length = length
    tries = 0
    while not done:
        tries += 1
        if tries > 5:
            break
        print "input_content: %s" % (input_content,)
        print "output_content: %s" % (output_content,)
        print "remaining_length: %s" % (remaining_length,)
        work_on = input_content[:length]

        if "[" not in work_on:
            if len(work_on) > remaining_length:
                output_content += work_on[:remaining_length-3] + "..."
            else:
                output_content += work_on

            done = True
            break

        tag_start = input_content.index("[")
        output_content += input_content[:tag_start+1]
        remaining_length -= tag_start
        input_content = input_content[tag_start+1:]

        if "]" not in input_content:
            if len(input_content) > remaining_length:
                output_content += input_content[:remaining_length-3] + "..."
            else:
                output_content += input_content
            output_content += "]"
            done = True
            break

        tag_end = input_content.index("]")
        output_content += input_content[:tag_end+1]
        input_content = input_content[tag_end+1:]

    #if len(content) > length:
    #    content = content[:length-3] + "..."
    return pm.render_to_html(output_content.encode('utf-8')).decode('utf-8')

from turbogears import widgets, validators
class PostMarkupExplanation(widgets.FormField):
    template = """
    <div xmlns:py="http://purl.org/kid/ns#"
        class="${field_class}"
        id="${field_id}"
    >
        <p>The text area above accepts Post Markup, a BBCode work-alike.</p>

        <pre>
[b]foo[/b]: <strong>foo</strong>
[i]foo[/i]: <em>foo</em>
[link]http://nxsy.org/[/link]: <a href="http://nxsy.org/">http://nxsy.org/</a> [nxsy.org]
[link http://nxsy.org/]Neil[/link]: <a href="http://nxsy.org/">Neil</a> [nxsy.org]
        </pre>

        <p>You can also use:</p>
        <pre>
[code python]
import foo
[/code]
        </pre>
    </div>
    """

def addCommentFields(wl):
    class CommentFormFieldsExtra(widgets.WidgetsList):
        comment = widgets.TextArea(label = "Your comment", rows=15, cols=45,
            validator=validators.NotEmpty,
        )
        explanation = PostMarkupExplanation()
    wl.extend(CommentFormFieldsExtra())

import commenting
class Commenting(commenting.Commenting):
    def convert(self, kw):
        pass
    def post_save(self, kw):
        pass
    
pm = postmarkup.PostMarkup().default_tags()
pm.add_tag('link', postmarkup.LinkTag, 'link')
pm.add_tag('quote', postmarkup.QuoteTag)
pm.add_tag('code', CodeTag)




import logging

import cherrypy

import turbogears
from turbogears import controllers, expose, validate, redirect, error_handler
from turbogears import identity, view, widgets, validators, paginate, flash
from turbogears import mochikit
from turbogears.database import session

from gibe import json, model
import routes

from datetime import datetime, timedelta
from gibe.util import slugify, excerptify, commaise, sanitise, display_entries

log = logging.getLogger("gibe.controllers")

from turbogears.decorator import weak_signature_decorator

from urllib2 import URLError

from gibe import plugin

blog_plugin_data = {}

from gibe.util import install_template_loader
from gibe.plugin import get_plugin_data, add_plugin_data

class Root(controllers.RootController):
    def __init__(self, *args, **kw):
        super(Root, self).__init__(*args, **kw)
        self._blogController = BlogController()

    @expose(template="gibe.templates.welcome")
    @add_plugin_data(template="gibe.templates.welcome")
    def index(self):
        import time
        log.debug("Happy TurboGears Controller Responding For Duty")
        return dict(now=time.ctime())

    @expose(template="gibe.templates.login")
    @add_plugin_data(template="gibe.templates.login")
    def _login(self, forward_url=None, previous_url=None, *args, **kw):

        if not identity.current.anonymous \
            and identity.was_login_attempted() \
            and not identity.get_identity_errors():
            raise redirect(forward_url)

        forward_url=None
        previous_url= cherrypy.request.path

        if identity.was_login_attempted():
            msg=_("The credentials you supplied were not correct or "
                   "did not grant access to this resource.")
        elif identity.get_identity_errors():
            msg=_("You must provide your credentials before accessing "
                   "this resource.")
        else:
            msg=_("Please log in.")
            forward_url= cherrypy.request.headers.get("Referer", "/")
        cherrypy.response.status=403
        return dict(message=msg, previous_url=previous_url, logging_in=True,
                    original_parameters=cherrypy.request.params,
                    forward_url=forward_url)

    @expose()
    @add_plugin_data()
    def logout(self):
        identity.current.logout()
        raise redirect("/")

    @expose()
    @add_plugin_data()
    def default(self, *args, **kw):
        blog = model.Blog.get_by(slug=args[0])
        if blog:
            return self._blogController.route(blog, *args[1:], **kw)
        raise cherrypy.NotFound()

def post_expand(kargs):
    if 'post' not in kargs:
        return kargs

    post = kargs.pop('post')
    kargs['year'] = post.creation_time.year
    kargs['month'] = "%02d" % (post.creation_time.month,)
    kargs['day'] = "%02d" % (post.creation_time.day,)
    kargs['slug'] = post.slug

    return kargs

def adminpost_expand(kargs):
    if 'post' not in kargs:
        return kargs

    post = kargs.pop('post')
    kargs['postid'] = post.post_id

    return kargs

def tag_expand(kargs):
    if 'tag' not in kargs:
        return kargs

    tag = kargs.pop('tag')
    kargs['tagname'] = tag.name

    return kargs

def tag_condition(environ, match_dict):
    if environ['PATH_INFO'].endswith('.html'):
        return False
    return True

def date_expand(d):
    if 'month' in d:
        d['month'] = "%02d" % (int(d['month']))
    if 'day' in d:
        d['day'] = "%02d" % (int(d['day']))
    if 'date' in d:
        del d['date']
    return d

def date_expand_day(d):
    if 'date' in d:
        if 'year' not in d:
            d['year'] = d['date'].year
        if 'month' not in d:
            d['month'] = d['date'].month
        if 'day' not in d:
            d['day'] = d['date'].day
    return date_expand(d)

def date_expand_month(d):
    if 'date' in d:
        if 'year' not in d:
            d['year'] = d['date'].year
        if 'month' not in d:
            d['month'] = d['date'].month
    return date_expand(d)

def date_expand_year(d):
    if 'date' in d:
        if 'year' not in d:
            d['year'] = d['date'].year
    return date_expand(d)
 
from tinymce import TinyMCE
class PostFormFields(widgets.WidgetsList):
    title = widgets.TextField(validator=validators.NotEmpty,
        label = "Title", attrs=dict(size="30"))
    content = TinyMCE(validator=validators.NotEmpty, label = "Content",
        mce_options = dict(
            mode = "exact",
            theme = "advanced",
            plugins = "fullscreen",
            relative_urls = False,
            theme_advanced_buttons2_add = "fullscreen",
            extended_valid_elements = "a[href|target|name]",
            paste_auto_cleanup_on_paste = True,
            paste_convert_headers_to_strong = True,
            paste_strip_class_attributes = "all",
            theme_advanced_buttons3 = "",
            remove_linebreaks = False,
            convert_newlines_to_brs = False,
            force_br_newlines = False,
        ),
        rows=25,
        cols=65,
    )

#post_form = widgets.TableForm(fields=PostFormFields(), submit_text="Post")

class AdminController(object):
    _post_form = None
    def post_form(self):
        if self._post_form:
            return self._post_form
        ws = plugin.post_edit_widgets(None, None)
        self._post_form = widgets.TableForm(fields=ws, submit_text="Post")
        return self._post_form

    @expose(template="genshi:gibe.templates.adminpost", content_type='text/html; charset=utf-8')
    @identity.require(identity.in_group("admin"))
    @add_plugin_data()
    @paginate('comments', limit = 25)
    def post(self, blog, postid, **kw):
        post = model.Post.get_by(post_id = postid)

        comments = list(post.comments)
        context = {'post': post}
        return dict(blog=blog, post_form = self.post_form(), post_form_values = plugin.post_edit_values(blog, post, context), post = post,
            comments = comments, approved_comments =
            post.approved_comments, mochikit = mochikit, post_form_params = plugin.post_edit_params(blog, post, context))

    @expose(template="genshi:gibe.templates.writepost", content_type='text/html; charset=utf-8')
    @identity.require(identity.in_group("admin"))
    @add_plugin_data()
    def writepost(self, blog, **kw):
        post = None
        context = {}
        return dict(blog=blog, post_form = self.post_form(), post_form_values = plugin.post_edit_values(blog, post, context), post_form_params = plugin.post_edit_params(blog, post, context))

    @error_handler(writepost)
    @identity.require(identity.in_group("admin"))
    @validate(form=post_form)
    @add_plugin_data()
    def newpost(self, blog, title, content, **kw):
        p = model.Post(title=title, content=content, author_id = 1, published = True, creation_time = datetime.now(), modification_time=datetime.now(), blog_id = blog.blog_id, slug = slugify(title, model.Post))
        kw['title'] = title
        kw['content'] = content
        p.excerpt = excerptify(p)
        context = {}
        plugin.post_edit_handle_after(blog, p, kw, context)
        session.save(p)
        raise routes.redirect_to('posts', post = p)

    @error_handler(post)
    @identity.require(identity.in_group("admin"))
    @validate(form=post_form)
    @add_plugin_data()
    def updatepost(self, blog, postid, title, content, **kw):
        p = model.Post.get_by(post_id = postid)
        p.title = title
        p.content = content
        context = {}
        kw['title'] = title
        kw['content'] = content
        p.excerpt = excerptify(p)
        plugin.post_edit_handle_after(blog, p, kw, context)
        session.update(p)
        raise routes.redirect_to('adminpost', post = p)

    @expose(template="genshi:gibe.templates.adminposts", content_type='text/html; charset=utf-8')
    @identity.require(identity.in_group("admin"))
    @add_plugin_data()
    @paginate('posts', limit = 25)
    def posts(self, blog, **kw):
        return dict(blog=blog, posts = model.Post.select(order_by=model.desc(model.Post.c.creation_time)))

    @expose(template="genshi:gibe.templates.admincomments", content_type='text/html; charset=utf-8')
    @identity.require(identity.in_group("admin"))
    @add_plugin_data()
    @paginate('comments', limit = 25)
    def comments(self, blog, **kw):
        return dict(blog=blog, comments =
        model.Comment.select(order_by=model.desc(model.Comment.c.posted_time)),
        mochikit = mochikit)

    @expose(template="genshi:gibe.templates.admincomments", content_type='text/html; charset=utf-8')
    @identity.require(identity.in_group("admin"))
    @add_plugin_data()
    @paginate('comments', limit = 25)
    def unapproved_comments(self, blog, **kw):
        return dict(blog=blog, comments =
            model.Comment.select(model.Comment.c.approved == False, order_by=model.desc(model.Comment.c.posted_time)),
            mochikit = mochikit)

    @expose(template="genshi:gibe.templates.adminindex", content_type='text/html; charset=utf-8')
    @identity.require(identity.in_group("admin"))
    @add_plugin_data()
    def index(self, blog, **kw):
        return dict(blog=blog, post_form = self.post_form())

    @identity.require(identity.in_group("admin"))
    @add_plugin_data()
    def stop_comments(self, blog, **kw):
        post = model.Post.get_by(post_id = kw['postid'])
        post.accept_comments = False
        session.save(post)
        raise routes.redirect_to('adminpost', post=post)

    @identity.require(identity.in_group("admin"))
    @add_plugin_data()
    def start_comments(self, blog, **kw):
        post = model.Post.get_by(post_id = kw['postid'])
        post.accept_comments = True
        session.save(post)
        raise routes.redirect_to('adminpost', post=post)
        
    @identity.require(identity.in_group("admin"))
    @add_plugin_data()
    def remove_comments(self, blog, post = None, **kw):
        comment_ids = kw.get('comment_id', [])
        import types
        if type(comment_ids) in types.StringTypes:
            comment_ids = [comment_ids]
        for comment_id in comment_ids:
            c = model.Comment.get_by(comment_id = comment_id)
            session.delete(c)
        if kw.get('return_url', None):
           raise cherrypy.HTTPRedirect(kw['return_url'])
        if post:
           raise routes.redirect_to('adminpost', post=post)
        raise routes.redirect_to("admincomments")

from gibe.feeds import FeedController

class TagController(object):
    @expose(template="genshi:gibe.templates.archives", content_type='text/html; charset=utf-8')
    @add_plugin_data()
    @paginate('posts', limit = 10)
    def tag(self, blog, tagname, **kw):
        t = model.Tag.get_by(name=tagname)
        if not t:
            raise cherrypy.NotFound()
    
        posts = t.recent_posts(limit = None)
        return dict(blog=blog, posts=posts, tag=t)

class BlogController(object):
    blog_is_root = False

    def __init__(self, *args, **kw):
        super(BlogController, self).__init__(*args, **kw)
        import re
        self._base_re = re.compile(r'^(?P<protocol>[a-zA-Z]+)://(?P<host>.*)' )
        from turbogears.startup import call_on_startup
        call_on_startup.append(self._create_mapper)

    def _redirect(self, url):
        raise cherrypy.HTTPRedirect(url, 302)

    def _create_mapper(self):
        self._mapper = routes.Mapper()
        mapper = self._mapper

        gibe_controllers['self'] = self

        maps = []
        maps.append(('frontpage', '', dict(controller='self', action='frontpage')))
        maps.append(('frontpage_redirect', 'redirect', dict(controller='self', action='frontpage_redirect')))
        maps.append(('index.html', 'index.html', dict(controller='self', action='frontpage')))
        maps.append(('posts', 'archives/:year/:month/:day/:slug', dict(controller='self', action='post', _filter=post_expand)))
        maps.append(('posts_by_id', ':(postid).html', dict(controller='self', action='post', requirements=dict(postid=r'\d+'))))
        maps.append(('archives_day', 'archives/:year/:month/:day/', dict(controller='self', action='archives_day', _filter=date_expand_day)))
        maps.append(('archives_month', 'archives/:year/:month/', dict(controller='self', action='archives_month', _filter=date_expand_month)))
        maps.append(('archives_month_deprecated', ':(year)-:(month).html', dict(controller='self', action='archives_month', _filter=date_expand_month, requirements=dict(year='\d{4,}', month='\d{1,2}'))))
        maps.append(('archives_year', 'archives/:year/', dict(controller='self', action='archives_year', _filter=date_expand_year)))
        maps.append(('archives', 'archives', dict(controller='self', action='archives')))
        maps.append(('writepost', 'admin/posts/write', dict(controller='adminController', action='writepost')))
        maps.append(('newpost', 'admin/posts/new', dict(controller='adminController', action='newpost')))
        maps.append(('updatepost', 'admin/posts/update/:postid', dict(controller='adminController', action='updatepost', _filter=adminpost_expand)))
        maps.append(('login', 'login', dict(controller='self', action='login')))
        maps.append(('logout', 'logout', dict(controller='self', action='logout')))
        maps.append(('email.js', 'email_js/:section', dict(controller='self', action='email_js', section = "start")))
        maps.append(('rss2.0.xml', 'rss2.0.xml', dict(controller='feedController', action='rss2_0')))
        maps.append(('atom1_0', 'atom1.0', dict(controller='feedController', action='atom1_0')))

        maps.append(('add_comment', 'form/add_comment', dict(controller='commentController', action='add_comment')))
        maps.append(('adminposts', 'admin/posts', dict(controller='adminController', action='posts')))
        maps.append(('stop_comments', 'admin/post/:postid/stop_comments', dict(controller='adminController', action='stop_comments', _filter=adminpost_expand)))
        maps.append(('start_comments', 'admin/post/:postid/start_comments', dict(controller='adminController', action='start_comments', _filter=adminpost_expand)))
        maps.append(('adminpost', 'admin/post/:postid', dict(controller='adminController', action='post', _filter=adminpost_expand)))
        maps.append(('remove_comments', 'admin/comments/remove', dict(controller='adminController', action='remove_comments', _filter=adminpost_expand)))
        maps.append(('comments', 'admin/comments', dict(controller='adminController', action='comments')))
        maps.append(('unapproved_comments', 'admin/unapproved_comments', dict(controller='adminController', action='unapproved_comments')))
        maps.append(('admin', 'admin', dict(controller='adminController', action='index')))

        plugin.map_routes(None, maps, gibe_controllers)

        for m in maps:
            mapper.connect(m[0], m[1], **m[2])

    @expose(template="genshi:gibe.templates.archives", content_type='text/html; charset=utf-8')
    @add_plugin_data(template="genshi:gibe.templates.archives")
    @paginate('posts', limit = 10)
    def archives(self, blog, **kw):
        posts = model.Post.select(order_by=model.desc(model.Post.c.creation_time))
        #display_entries = []
        #last = None
        #for entry in last_entries:
        #    this = datetime(year=entry.creation_time.year, month=entry.creation_time.month, day=entry.creation_time.day)
        #    if last != this:
        #        last = this
        #        last_list = (last, [])
        #        display_entries.append(last_list)
        #
        #    last_list[1].append(entry)
        return dict(blog=blog, link = routes.url_for('frontpage'), posts = posts)
    archives.exposed = False

    @expose(template="genshi:gibe.templates.archives", content_type='text/html; charset=utf-8')
    @add_plugin_data(template="genshi:gibe.templates.archives")
    @paginate('posts', limit = 10)
    def archives_day(self, blog, year, **kw):
        st = datetime(year = int(year), month = int(kw.get('month', 1)), day = int(kw.get('day', 1)))
        day = kw.get('day')
        month = kw.get('month')
        if day:
            et = st + timedelta(days=1)
        elif month:
            et = st + timedelta(days=32)
            et = datetime(et.year, month=et.month, day=1)
        else:
            et = st + timedelta(days=366)
            et = datetime(et.year, month=1, day=1)

        posts = model.Post.select(model.Post.c.creation_time.between(st, et), order_by=model.desc(model.Post.c.creation_time))
        return dict(blog=blog, link = routes.url_for('frontpage'), posts = posts)
    archives_day.exposed = False
    archives_month = archives_day
    archives_year = archives_day

    @expose(content_type="text/javascript")
    def email_js(self, blog, section, **kw):
        #log.info("section=%s" % (section,))
        if section == "end":
            return """document.write('</a>');"""
        else:
            return """document.write('<a href="mailto:nbm' + '@' + 'mithrandr.moria.org">');"""
    email_js.exposed = False

    def _configure_mapper(self, blog):

        base = cherrypy.request.base
        d = self._base_re.match(base).groupdict()
        host = d['host']
        proto = d['protocol']

        config = routes.request_config()
        config.mapper = self._mapper
        host = cherrypy.config.get('gibe.hostname', host)
        config.host = host
        config.protocol = proto
        config.redirect = self._redirect

        path = "/"

        # Pure evil...
        if not self.blog_is_root:
            path = '/%s' % (blog.slug)

        path = turbogears.url(path)
        if path[-1] == "/":
            path = path[:-1]

        from cherrypy.filters.wsgiappfilter import make_environ
        environ = make_environ()
        #logging.info(str(environ))
        environ['SERVER_PORT'] = str(cherrypy.config.get('gibe.port', cherrypy.config.get('server.socket_port')))
        environ['SCRIPT_NAME'] = path
        #logging.info("SCRIPT_NAME is %s" % (environ['SCRIPT_NAME'],))
        config.environ = environ

        return config

    def route(self, blog, *args, **kw):
        cherrypy.request.blog = blog
        install_template_loader()
        config = self._configure_mapper(blog)
        import os.path
        if len(args):
            path = "/" + os.path.join(*args)
        else:
            path = "/"
        mapper_dict = self._mapper.match(path)
        if not mapper_dict:
            raise cherrypy.NotFound()
            return str(dict(path=path, mapper_dict=mapper_dict, args=args, blog=blog))

        controller = gibe_controllers[mapper_dict['controller']]
        try:
            action = getattr(controller, mapper_dict['action'])
        except AttributeError:
            raise cherrypy.NotFound()

        kw['blog'] = blog
        del mapper_dict['controller']
        kw.update(mapper_dict)
        return action(**kw)

    @expose(template="genshi:gibe.templates.post", content_type='text/html; charset=utf-8')
    @add_plugin_data(template="genshi:gibe.templates.post")
    def post(self, blog, **kw):
        if 'postid' in kw:
            post = model.Post.get_by(post_id = kw['postid'])
        if 'slug' in kw:
            post = model.Post.get_by(slug = kw['slug'])

        id = post.post_id

        previous_post = model.Post.select(model.Post.c.post_id < id, order_by=model.desc(model.Post.c.post_id), limit = 1)
        next_post = model.Post.select(model.Post.c.post_id > id, order_by=model.asc(model.Post.c.post_id), limit = 1)
        if previous_post:
            previous_post = previous_post[0]
        if next_post:
            next_post = next_post[0]

        import commenting
        context = {}
        return dict(blog=blog,
            post = post,
            previous_post = previous_post,
            next_post = next_post,
            url_for = routes.url_for,
            comment_form = commenting.comment_form,
            accept_comments = post.accept_comments
        )

    @error_handler(post)
    #@validate(form=commenting.comment_form)
    def add_comment(self, blog, **kw):
        return commenting.commentController.add_comment(blog, **kw)

    @expose(template="genshi:gibe.templates.frontpage", content_type='text/html; charset=utf-8')
    @add_plugin_data(template="genshi:gibe.templates.frontpage")
    @paginate('posts', limit = 25)
    def frontpage(self, blog, **kw):
        posts = model.Post.select(order_by=model.desc(model.Post.c.creation_time))
        return dict(blog=blog, link = routes.url_for('frontpage'), posts = posts)

    @expose()
    def frontpage_redirect(self, blog, **kw):
        return routes.redirect_to('frontpage')

    @expose(template="genshi:gibe.templates.login", content_type='text/html; charset=utf-8')
    @add_plugin_data()
    def login(self, blog, forward_url=None, previous_url=None, *args, **kw):

        if not identity.current.anonymous \
            and identity.was_login_attempted() \
            and not identity.get_identity_errors():
            raise redirect(forward_url)

        forward_url=None
        previous_url= cherrypy.request.path

        if identity.was_login_attempted():
            msg=_("The credentials you supplied were not correct or "
                   "did not grant access to this resource.")
        elif identity.get_identity_errors():
            msg=_("You must provide your credentials before accessing "
                   "this resource.")
        else:
            msg=_("Please log in.")
            forward_url= cherrypy.request.headers.get("Referer", "/")
        cherrypy.response.status=403
        return dict(blog=blog, message=msg, previous_url=previous_url, logging_in=True,
                    original_parameters=cherrypy.request.params,
                    forward_url=forward_url)
    login.exposed = False

    @expose()
    @add_plugin_data()
    def logout(self, blog, *args, **kw):
        identity.current.logout()
        raise redirect("/")
    logout.exposed = False

class RootBlogController(BlogController, controllers.RootController):
    blog_is_root = True

    def __init__(self, blog, *args, **kw):
        super(RootBlogController, self).__init__(*args, **kw)
        self.blog = blog

    @expose()
    def default(self, *args, **kw):
        return self.route(self.blog, *args, **kw)

    @expose()
    def index(self, *args, **kw):
        return self.route(self.blog, *args, **kw)

import urlparse

def comment_class(comment):
    if comment.approved:
        return "gibe_comment_approved"
    return "gibe_comment_unapproved"

from genshi.core import Attrs, QName, Stream, stripentities
from genshi.core import DOCTYPE, START, END, START_NS, END_NS, TEXT, \
                        START_CDATA, END_CDATA, PI, COMMENT
def myET(element):
    """Convert a given ElementTree element to a markup stream."""
    tag_name = QName(element.tag.lstrip('{'))
    attrs = Attrs((QName(k), v) for k, v in element.items())

    yield START, (tag_name, attrs), (None, -1, -1)
    if element.text:
        yield TEXT, element.text, (None, -1, -1)
    for child in element.getchildren():
        for item in myET(child):
            yield item
    yield END, tag_name, (None, -1, -1)
    if element.tail:
        yield TEXT, element.tail, (None, -1, -1)

def add_usual_suspects(d):
    d['url_for'] = routes.url_for
    d['production'] = (cherrypy.config.get('server.environment', '') == 'production')
    d['commaise'] = commaise
    # has the query string
    d['browser_url'] = cherrypy.request.browser_url


    url_parts = list(urlparse.urlparse(cherrypy.request.browser_url))
    url_parts[4] = None
    # no query string
    d['base_url'] = urlparse.urlunparse(url_parts)

    d['comment_class'] = comment_class
    d['display_entries'] = display_entries
    d['ET'] = myET

    return

view.variable_providers.append(add_usual_suspects)

gibe_controllers = {}
def getControllers():
    import commenting
    return dict(
        adminController=AdminController(),
        feedController=FeedController(),
        #tagController=TagController(),
        commentController=commenting.CommentController(),
    )
gibe_controllers.update(getControllers())

